@@ -11,7 +11,7 @@ use strict;
 use vars qw($VERSION);
 use Carp ();
 
-$VERSION = '1.18';
+$VERSION = '1.20';
 
 sub PV  { 0 }
 sub IV  { 1 }
@@ -25,6 +25,7 @@ my $ERRORS = {
         # PP and XS
         1000 => "INI - constructor failed",
         1001 => "sep_char is equal to quote_char or escape_char",
+        1002 => "INI - allow_whitespace with escape_char or quote_char SP or TAB",
 
         2010 => "ECR - QUO char inside quotes followed by CR not part of EOL",
         2011 => "ECR - Characters after end of quoted field",
@@ -64,6 +65,7 @@ my $ERRORS = {
 
 
 my $last_new_error = '';
+my $last_new_err_num;
 
 my %def_attr = (
     quote_char          => '"',
@@ -144,12 +146,22 @@ sub new {
 
     for my $prop (keys %$attr) { # if invalid attr, return undef
         unless ($prop =~ /^[a-z]/ && exists $def_attr{$prop}) {
-            $last_new_error = "Unknown attribute '$prop'";
+            $last_new_error = "INI - Unknown attribute '$prop'";
             return;
         }
         $self->{$prop} = $attr->{$prop};
     }
 
+    if ( $self->{allow_whitespace} and
+           ( defined $self->{quote_char}  && $self->{quote_char}  =~ m/^[ \t]$/ ) 
+           ||
+           ( defined $self->{escape_char} && $self->{escape_char} =~ m/^[ \t]$/ )
+    ) {
+       $last_new_error = "INI - allow_whitespace with escape_char or quote_char SP or TAB";
+       $last_new_err_num = 1002;
+       return;
+    }
+
     $last_new_error = '';
 
     defined $\ and $self->{eol} = $\;
@@ -180,7 +192,7 @@ sub error_diag {
     my @diag = (0, $last_new_error, 0);
 
     unless ($self and ref $self) {	# Class method or direct call
-        $last_new_error and $diag[0] = 1000;
+        $last_new_error and $diag[0] = defined $last_new_err_num ? $last_new_err_num : 1000;
     }
     elsif ( $self->isa (__PACKAGE__) and defined $self->{_ERROR_DIAG} ) {
         @diag = ( 0 + $self->{_ERROR_DIAG}, $ERRORS->{ $self->{_ERROR_DIAG} } );
@@ -274,7 +286,8 @@ sub _combine {
         }
     }
 
-    $self->{_STRING} = \do { join($sep, @part) . $self->{eol} };
+#    $self->{_STRING} = \do { join($sep, @part) . $self->{eol} };
+    $self->{_STRING} = \do { join($sep, @part) . ( defined $self->{eol} ? $self->{eol} : '' ) };
     $self->{_STATUS} = 1;
 
     return $self->{_STATUS};
@@ -289,7 +302,7 @@ my %allow_eol = ("\r" => 1, "\r\n" => 1, "\n" => 1, "" => 1);
 sub _parse {
     my ($self, $line) = @_;
 
-    @{$self}{qw/_STRING _FIELDS _STATUS _ERROR_INPUT/} = ($line, undef, 0, $line);
+    @{$self}{qw/_STRING _FIELDS _STATUS _ERROR_INPUT/} = ( \do{ defined $line ? "$line" : undef }, undef, 0, $line );
 
     return 0 if(!defined $line);
 
@@ -342,7 +355,7 @@ sub _parse {
 
     my $utf8 = 1 if utf8::is_utf8( $line ); # if UTF8 marked, flag on.
 
-    for my $col ($line =~ /$re_split/g) {
+    for my $col ( $line =~ /$re_split/g ) {
 
         if ($keep_meta_info) {
             $flag = 0x0000;
@@ -382,8 +395,11 @@ sub _parse {
             $flag |= IS_QUOTED if ($keep_meta_info);
             $col = $1;
 
-            if ($col =~ $re_in_quot_esp1) {
+            my $flga_in_quot_esp;
+            while ( $col =~ /$re_in_quot_esp1/g ) {
                 my $str = $1;
+                $flga_in_quot_esp = 1;
+
                 if ($str !~ $re_in_quot_esp2) {
                     unless ($self->{allow_loose_escapes}) {
                         $self->_set_error_diag( 2025, $pos - 2 ); # Needless ESC in quoted field
@@ -394,8 +410,12 @@ sub _parse {
                         $col =~ s/\Q$esc\E(.)/$1/g;
                     }
                 }
+
             }
-            else {
+
+            last unless ( $palatable );
+
+            unless ( $flga_in_quot_esp ) {
                 if ($col =~ /(?<!\Q$esc\E)\Q$esc\E/) {
                     $self->_set_error_diag( 4002, $pos - 1 ); # No escaped ESC in quoted field
                     $palatable = 0;
@@ -553,7 +573,7 @@ sub getline {
 
     my $line = $io->getline();
     my $quot = $self->{quote_char};
-    my $re   = qr/(?:\Q$quot\E)/;
+    my $re   = $self->binary ? qr/(?:\Q$quot\E)(?!0)/ : qr/(?:\Q$quot\E)/;
 
     $line .= $io->getline() while ( defined $line and scalar(my @list = $line =~ /$re/g) % 2 and !eof($io) );
 
@@ -730,7 +750,7 @@ sub _set_error_diag {
 ################################################################################
 
 BEGIN {
-    for my $method (qw/quote_char escape_char sep_char always_quote binary allow_whitespace
+    for my $method (qw/sep_char always_quote binary
                         keep_meta_info allow_loose_quotes allow_loose_escapes verbatim blank_is_undef/) {
         eval qq|
             sub $method {
@@ -742,6 +762,42 @@ BEGIN {
 }
 
 
+sub quote_char {
+    my $self = shift;
+    if ( @_ ) {
+        my $qc = shift;
+        defined $qc && $qc =~ m/^[ \t]$/ && $self->{allow_whitespace} and Carp::croak( $self->SetDiag(1002) );
+        $self->{quote_char} = $qc;
+    }
+    $self->{quote_char};
+}
+
+
+sub escape_char {
+    my $self = shift;
+    if ( @_ ) {
+        my $es = shift;
+        defined $es && $es =~ m/^[ \t]$/ && $self->{allow_whitespace} and Carp::croak( $self->SetDiag(1002) );
+        $self->{escape_char} = $es;
+    }
+    $self->{escape_char};
+}
+
+
+sub allow_whitespace {
+    my $self = shift;
+    if ( @_ ) {
+        my $aw = shift;
+        $aw and
+            (defined $self->{quote_char}  && $self->{quote_char}  =~ m/^[ \t]$/) ||
+            (defined $self->{escape_char} && $self->{escape_char} =~ m/^[ \t]$/)
+                and Carp::croak ($self->SetDiag (1002));
+        $self->{allow_whitespace} = $aw;
+    }
+    $self->{allow_whitespace};
+}
+
+
 sub eol {
     $_[0]->{eol} = defined $_[1] ? $_[1] : '' if ( @_ > 1 );
     $_[0]->{eol};
@@ -882,12 +938,16 @@ See also L<Text::CSV_XS/CAVEATS>
 =item allow_whitespace
 
 When this option is set to true, whitespace (TAB's and SPACE's)
-surrounding the separation character is removed when parsing. So
-lines like:
+surrounding the separation character is removed when parsing. If
+either TAB or SPACE is one of the three major characters C<sep_char>,
+C<quote_char>, or C<escape_char> it will not be considered whitespace.
+
+So lines like:
 
   1 , "foo" , bar , 3 , zapp
 
 are now correctly parsed, even though it violates the CSV specs.
+
 Note that B<all> whitespace is stripped from start and end of each
 field. That would make is more a I<feature> than a way to be able
 to parse bad CSV lines, as
@@ -1088,7 +1148,7 @@ the fail reason available through the C<error_diag ()> method.
 
 C<error_diag ()> will return a string like
 
- "Unknown attribute 'ecs_char'"
+ "INI - Unknown attribute 'ecs_char'"
 
 =head2 combine
 
@@ -1378,6 +1438,11 @@ Currently these errors are available:
 The separation character cannot be equal to either the quotation character
 or the escape character, as that will invalidate all parsing rules.
 
+=item 1002 "INI - allow_whitespace with escape_char or quote_char SP or TAB"
+
+Using C<allow_whitespace> when either C<escape_char> or C<quote_char> is
+equal to SPACE or TAB is too ambiguous to allow.
+
 =item 2010 "ECR - QUO char inside quotes followed by CR not part of EOL"
 
 =item 2011 "ECR - Characters after end of quoted field"
@@ -1440,7 +1505,7 @@ Text::CSV was written by E<lt>alan[at]mfgrtl.comE<gt>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2005-2008 by Makamaka Hannyaharamitu, E<lt>makamaka[at]cpan.orgE<gt>
+Copyright 2005-2009 by Makamaka Hannyaharamitu, E<lt>makamaka[at]cpan.orgE<gt>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
